@@ -1,9 +1,16 @@
-/*
- * This file is part of the zlog Library.
- *
- * Copyright (C) 2011 by Hardy Simpson <HardySimpson1984@gmail.com>
- *
- * Licensed under the LGPL v2.1, see the file COPYING in base directory.
+/* Copyright (c) Hardy Simpson
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 #ifndef __zc_xplatform_h
 #define __zc_xplatform_h
@@ -45,6 +52,9 @@
 #if defined(__APPLE__) && !defined(MAC_OS_X_VERSION_10_6)
 #define zlog_fstat fstat64
 #define zlog_stat stat64
+#elif defined(_WIN32)
+#define zlog_fstat _fstat
+#define zlog_stat _stat
 #else
 #define zlog_fstat fstat
 #define zlog_stat stat
